@@ -10,6 +10,7 @@ from tempfile import NamedTemporaryFile
 from typing import Dict, Optional, Union, List, Tuple
 import logging
 
+from faiss import extract_index_ivf
 import faiss
 import fsspec
 import numpy as np
@@ -155,3 +156,30 @@ def parallel_download_indices_from_remote(
     with ThreadPool(min(16, len(indices_file_paths))) as pool:
         for _ in pool.imap_unordered(partial(_download_one, fs=fs), src_dest_paths):
             pass
+
+
+def initialize_direct_map(index: faiss.Index) -> None:
+    nested_index = extract_index_ivf(index) if isinstance(index, faiss.swigfaiss.IndexPreTransform) else index
+
+    # Make direct map is only implemented for IndexIVF and IndexBinaryIVF, see built file faiss/swigfaiss.py
+    if isinstance(nested_index, (faiss.swigfaiss.IndexIVF, faiss.swigfaiss.IndexBinaryIVF)):
+        nested_index.make_direct_map()
+
+
+def save_index(index: faiss.Index, root_dir: str, index_filename: str) -> str:
+    """Save index"""
+    fs = fsspec.core.url_to_fs(root_dir, use_listings_cache=False)[0]
+    fs.mkdirs(root_dir, exist_ok=True)
+    output_index_path = os.path.join(root_dir, index_filename)
+    with fsspec.open(output_index_path, "wb").open() as f:
+        faiss.write_index(index, faiss.PyCallbackIOWriter(f.write))
+    return output_index_path
+
+
+def load_index(index_src_path: str, index_dst_path: str) -> faiss.Index:
+    fs = fsspec.core.url_to_fs(index_src_path, use_listings_cache=False)[0]
+    try:
+        fs.get(index_src_path, index_dst_path)
+    except Exception as e:
+        raise Exception(f"Failed to download index from {index_src_path} to {index_dst_path}") from e
+    return faiss.read_index(index_dst_path)
